@@ -23,14 +23,13 @@
 #include <gst/video/video.h>
 #include <gst/gst-i18n-plugin.h>
 
-static GstElementDetails gst_pngdec_details = {
-  "PNG decoder",
-  "Codec/Decoder/Image",
-  "Decode a png video frame to a raw image",
-  "Wim Taymans <wim@fluendo.com>",
-};
+static const GstElementDetails gst_pngdec_details =
+GST_ELEMENT_DETAILS ("PNG image decoder",
+    "Codec/Decoder/Image",
+    "Decode a png video frame to a raw image",
+    "Wim Taymans <wim@fluendo.com>");
 
-GST_DEBUG_CATEGORY (pngdec_debug);
+GST_DEBUG_CATEGORY_STATIC (pngdec_debug);
 #define GST_CAT_DEFAULT pngdec_debug
 
 static void gst_pngdec_base_init (gpointer g_class);
@@ -54,6 +53,7 @@ static GstFlowReturn gst_pngdec_caps_create_and_set (GstPngDec * pngdec);
 static void gst_pngdec_task (GstPad * pad);
 static GstFlowReturn gst_pngdec_chain (GstPad * pad, GstBuffer * buffer);
 static gboolean gst_pngdec_sink_event (GstPad * pad, GstEvent * event);
+static gboolean gst_pngdec_sink_setcaps (GstPad * pad, GstCaps * caps);
 
 static GstElementClass *parent_class = NULL;
 
@@ -116,19 +116,18 @@ gst_pngdec_class_init (GstPngDecClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstelement_class = (GstElementClass *) klass;
 
-  parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
+  parent_class = g_type_class_peek_parent (klass);
 
   gstelement_class->change_state = gst_pngdec_change_state;
 
   GST_DEBUG_CATEGORY_INIT (pngdec_debug, "pngdec", 0, "PNG image decoder");
 }
 
-
 static void
 gst_pngdec_init (GstPngDec * pngdec)
 {
-  pngdec->sinkpad = gst_pad_new_from_template (gst_static_pad_template_get
-      (&gst_pngdec_sink_pad_template), "sink");
+  pngdec->sinkpad =
+      gst_pad_new_from_static_template (&gst_pngdec_sink_pad_template, "sink");
   gst_pad_set_activate_function (pngdec->sinkpad, gst_pngdec_sink_activate);
   gst_pad_set_activatepush_function (pngdec->sinkpad,
       gst_pngdec_sink_activate_push);
@@ -136,15 +135,14 @@ gst_pngdec_init (GstPngDec * pngdec)
       gst_pngdec_sink_activate_pull);
   gst_pad_set_chain_function (pngdec->sinkpad, gst_pngdec_chain);
   gst_pad_set_event_function (pngdec->sinkpad, gst_pngdec_sink_event);
+  gst_pad_set_setcaps_function (pngdec->sinkpad, gst_pngdec_sink_setcaps);
   gst_element_add_pad (GST_ELEMENT (pngdec), pngdec->sinkpad);
 
-  pngdec->srcpad = gst_pad_new_from_template (gst_static_pad_template_get
-      (&gst_pngdec_src_pad_template), "src");
+  pngdec->srcpad =
+      gst_pad_new_from_static_template (&gst_pngdec_src_pad_template, "src");
   gst_pad_use_fixed_caps (pngdec->srcpad);
   gst_element_add_pad (GST_ELEMENT (pngdec), pngdec->srcpad);
 
-  /* gst_pad_set_chain_function (pngdec->sinkpad, gst_pngdec_chain); */
-
   pngdec->buffer_out = NULL;
   pngdec->png = NULL;
   pngdec->info = NULL;
@@ -157,6 +155,11 @@ gst_pngdec_init (GstPngDec * pngdec)
   pngdec->bpp = -1;
   pngdec->fps_n = 0;
   pngdec->fps_d = 1;
+
+  pngdec->in_timestamp = GST_CLOCK_TIME_NONE;
+  pngdec->in_duration = GST_CLOCK_TIME_NONE;
+
+  gst_segment_init (&pngdec->segment, GST_FORMAT_UNDEFINED);
 }
 
 static void
@@ -218,7 +221,7 @@ user_endrow_callback (png_structp png_ptr, png_bytep new_row,
   if (GST_IS_BUFFER (pngdec->buffer_out)) {
     size_t offset = row_num * GST_ROUND_UP_4 (pngdec->rowbytes);
 
-    GST_LOG ("got row %d, copying in buffer %p at offset %d", row_num,
+    GST_LOG ("got row %u, copying in buffer %p at offset %d", (guint) row_num,
         pngdec->buffer_out, offset);
     memcpy (GST_BUFFER_DATA (pngdec->buffer_out) + offset, new_row,
         pngdec->rowbytes);
@@ -229,6 +232,29 @@ user_endrow_callback (png_structp png_ptr, png_bytep new_row,
   }
 }
 
+static gboolean
+buffer_clip (GstPngDec * dec, GstBuffer * buffer)
+{
+  gboolean res = TRUE;
+  gint64 cstart, cstop;
+
+  if ((!GST_CLOCK_TIME_IS_VALID (GST_BUFFER_TIMESTAMP (buffer))) ||
+      (!GST_CLOCK_TIME_IS_VALID (GST_BUFFER_DURATION (buffer))) ||
+      (dec->segment.format != GST_FORMAT_TIME))
+    goto beach;
+
+  if ((res = gst_segment_clip (&dec->segment, GST_FORMAT_TIME,
+              GST_BUFFER_TIMESTAMP (buffer),
+              GST_BUFFER_TIMESTAMP (buffer) + GST_BUFFER_DURATION (buffer),
+              &cstart, &cstop))) {
+    GST_BUFFER_TIMESTAMP (buffer) = cstart;
+    GST_BUFFER_DURATION (buffer) = cstop - cstart;
+  }
+
+beach:
+  return res;
+}
+
 static void
 user_end_callback (png_structp png_ptr, png_infop info)
 {
@@ -236,11 +262,40 @@ user_end_callback (png_structp png_ptr, png_infop info)
 
   pngdec = GST_PNGDEC (png_ptr->io_ptr);
 
-  GST_LOG ("and we are done reading this image, pushing it and setting eos");
+  GST_LOG_OBJECT (pngdec, "and we are done reading this image");
 
-  /* Push our buffer and then EOS */
-  pngdec->ret = gst_pad_push (pngdec->srcpad, pngdec->buffer_out);
-  pngdec->ret = gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+  if (!pngdec->buffer_out)
+    return;
+
+  if (GST_CLOCK_TIME_IS_VALID (pngdec->in_timestamp))
+    GST_BUFFER_TIMESTAMP (pngdec->buffer_out) = pngdec->in_timestamp;
+  if (GST_CLOCK_TIME_IS_VALID (pngdec->in_duration))
+    GST_BUFFER_DURATION (pngdec->buffer_out) = pngdec->in_duration;
+
+  /* buffer clipping */
+  if (buffer_clip (pngdec, pngdec->buffer_out)) {
+    /* Push our buffer and then EOS if needed */
+    GST_LOG_OBJECT (pngdec, "pushing buffer with ts=%" GST_TIME_FORMAT,
+        GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (pngdec->buffer_out)));
+
+    pngdec->ret = gst_pad_push (pngdec->srcpad, pngdec->buffer_out);
+  } else {
+    GST_LOG_OBJECT (pngdec, "dropped decoded buffer");
+    gst_buffer_unref (pngdec->buffer_out);
+  }
+  pngdec->buffer_out = NULL;
+
+  if (pngdec->framed) {
+    /* Reset ourselves for the next frame */
+    gst_pngdec_libpng_clear (pngdec);
+    gst_pngdec_libpng_init (pngdec);
+    GST_LOG_OBJECT (pngdec, "setting up callbacks for next frame");
+    png_set_progressive_read_fn (pngdec->png, pngdec,
+        user_info_callback, user_endrow_callback, user_end_callback);
+  } else {
+    GST_LOG_OBJECT (pngdec, "sending EOS");
+    pngdec->ret = gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+  }
 }
 
 static void
@@ -283,7 +338,6 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
   GstFlowReturn ret = GST_FLOW_OK;
   GstCaps *caps = NULL, *res = NULL;
   GstPadTemplate *templ = NULL;
-  GstEvent *new_seg = NULL;
   gint bpc = 0, color_type;
   png_uint_32 width, height;
 
@@ -294,7 +348,8 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
 
   /* We don't handle 16 bits per color, strip down to 8 */
   if (bpc == 16) {
-    GST_LOG ("this is a 16 bits per channel PNG image, strip down to 8 bits");
+    GST_LOG_OBJECT (pngdec,
+        "this is a 16 bits per channel PNG image, strip down to 8 bits");
     png_set_strip_16 (pngdec->png);
   }
 
@@ -305,6 +360,23 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
   if (color_type == PNG_COLOR_TYPE_RGB_ALPHA)
     png_set_bgr (pngdec->png);
 
+  /* Gray scale converted to RGB and upscaled to 8 bits */
+  if ((color_type == PNG_COLOR_TYPE_GRAY_ALPHA) ||
+      (color_type == PNG_COLOR_TYPE_GRAY)) {
+    GST_LOG_OBJECT (pngdec, "converting grayscale png to RGB");
+    png_set_gray_to_rgb (pngdec->png);
+    if (bpc < 8) {              /* Convert to 8 bits */
+      GST_LOG_OBJECT (pngdec, "converting grayscale image to 8 bits");
+      png_set_gray_1_2_4_to_8 (pngdec->png);
+    }
+  }
+
+  /* Palette converted to RGB */
+  if (color_type == PNG_COLOR_TYPE_PALETTE) {
+    GST_LOG_OBJECT (pngdec, "converting palette png to RGB");
+    png_set_palette_to_rgb (pngdec->png);
+  }
+
   /* Update the info structure */
   png_read_update_info (pngdec->png, pngdec->info);
 
@@ -316,21 +388,22 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
   pngdec->width = width;
   pngdec->height = height;
 
-  GST_LOG ("this is a %dx%d PNG image", pngdec->width, pngdec->height);
+  GST_LOG_OBJECT (pngdec, "this is a %dx%d PNG image", pngdec->width,
+      pngdec->height);
 
   switch (pngdec->color_type) {
     case PNG_COLOR_TYPE_RGB:
-      GST_LOG ("we have no alpha channel, depth is 24 bits");
+      GST_LOG_OBJECT (pngdec, "we have no alpha channel, depth is 24 bits");
       pngdec->bpp = 24;
       break;
     case PNG_COLOR_TYPE_RGB_ALPHA:
-      GST_LOG ("we have an alpha channel, depth is 32 bits");
+      GST_LOG_OBJECT (pngdec, "we have an alpha channel, depth is 32 bits");
       pngdec->bpp = 32;
       break;
     default:
       GST_ELEMENT_ERROR (pngdec, STREAM, NOT_IMPLEMENTED, (NULL),
-          ("pngdec does not support grayscale or paletted data yet"));
-      ret = GST_FLOW_ERROR;
+          ("pngdec does not support this color type"));
+      ret = GST_FLOW_NOT_SUPPORTED;
       goto beach;
   }
 
@@ -345,19 +418,20 @@ gst_pngdec_caps_create_and_set (GstPngDec * pngdec)
   res = gst_caps_intersect (caps, gst_pad_template_get_caps (templ));
 
   gst_caps_unref (caps);
+  gst_object_unref (templ);
 
-  if (!gst_pad_set_caps (pngdec->srcpad, res)) {
-    ret = GST_FLOW_ERROR;
-  }
+  if (!gst_pad_set_caps (pngdec->srcpad, res))
+    ret = GST_FLOW_NOT_NEGOTIATED;
 
   GST_DEBUG_OBJECT (pngdec, "our caps %" GST_PTR_FORMAT, res);
 
   gst_caps_unref (res);
 
   /* Push a newsegment event */
-  new_seg = gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME, 0, -1, 0);
-  if (GST_IS_EVENT (new_seg)) {
-    gst_pad_push_event (pngdec->srcpad, new_seg);
+  if (pngdec->need_newsegment) {
+    gst_pad_push_event (pngdec->srcpad,
+        gst_event_new_new_segment (FALSE, 1.0, GST_FORMAT_TIME, 0, -1, 0));
+    pngdec->need_newsegment = FALSE;
   }
 
 beach:
@@ -430,13 +504,15 @@ gst_pngdec_task (GstPad * pad)
   return;
 
 pause:
-  GST_LOG_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
-  gst_pad_pause_task (pngdec->sinkpad);
-  if (GST_FLOW_IS_FATAL (ret)) {
-    gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
-    GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
-        (_("Internal data stream error.")),
-        ("stream stopped, reason %s", gst_flow_get_name (ret)));
+  {
+    GST_LOG_OBJECT (pngdec, "pausing task, reason %s", gst_flow_get_name (ret));
+    gst_pad_pause_task (pngdec->sinkpad);
+    if (GST_FLOW_IS_FATAL (ret)) {
+      gst_pad_push_event (pngdec->srcpad, gst_event_new_eos ());
+      GST_ELEMENT_ERROR (pngdec, STREAM, FAILED,
+          (_("Internal data stream error.")),
+          ("stream stopped, reason %s", gst_flow_get_name (ret)));
+    }
   }
 }
 
@@ -446,50 +522,137 @@ gst_pngdec_chain (GstPad * pad, GstBuffer * buffer)
   GstPngDec *pngdec;
   GstFlowReturn ret = GST_FLOW_OK;
 
-  pngdec = GST_PNGDEC (GST_OBJECT_PARENT (pad));
+  pngdec = GST_PNGDEC (gst_pad_get_parent (pad));
 
-  if (!pngdec->setup) {
-    GST_LOG ("we are not configured yet");
-    ret = GST_FLOW_WRONG_STATE;
-    goto beach;
-  }
+  GST_LOG_OBJECT (pngdec, "Got buffer, size=%u", GST_BUFFER_SIZE (buffer));
+
+  if (G_UNLIKELY (!pngdec->setup))
+    goto not_configured;
 
   /* Something is going wrong in our callbacks */
-  if (pngdec->ret != GST_FLOW_OK) {
-    ret = pngdec->ret;
+  ret = pngdec->ret;
+  if (G_UNLIKELY (ret != GST_FLOW_OK)) {
+    GST_WARNING_OBJECT (pngdec, "we have a pending return code of %d", ret);
     goto beach;
   }
 
   /* Let libpng come back here on error */
   if (setjmp (png_jmpbuf (pngdec->png))) {
+    GST_WARNING_OBJECT (pngdec, "error during decoding");
     ret = GST_FLOW_ERROR;
     goto beach;
   }
 
+  pngdec->in_timestamp = GST_BUFFER_TIMESTAMP (buffer);
+  pngdec->in_duration = GST_BUFFER_DURATION (buffer);
+
   /* Progressive loading of the PNG image */
   png_process_data (pngdec->png, pngdec->info, GST_BUFFER_DATA (buffer),
       GST_BUFFER_SIZE (buffer));
 
+  /* grab new return code */
+  ret = pngdec->ret;
+
   /* And release the buffer */
   gst_buffer_unref (buffer);
 
 beach:
+  gst_object_unref (pngdec);
+
   return ret;
+
+  /* ERRORS */
+not_configured:
+  {
+    GST_LOG_OBJECT (pngdec, "we are not configured yet");
+    ret = GST_FLOW_WRONG_STATE;
+    goto beach;
+  }
+}
+
+static gboolean
+gst_pngdec_sink_setcaps (GstPad * pad, GstCaps * caps)
+{
+  GstStructure *s;
+  GstPngDec *pngdec;
+  gint num, denom;
+
+  pngdec = GST_PNGDEC (gst_pad_get_parent (pad));
+
+  s = gst_caps_get_structure (caps, 0);
+  if (gst_structure_get_fraction (s, "framerate", &num, &denom)) {
+    GST_DEBUG_OBJECT (pngdec, "framed input");
+    pngdec->framed = TRUE;
+    pngdec->fps_n = num;
+    pngdec->fps_d = denom;
+  } else {
+    pngdec->framed = FALSE;
+    pngdec->fps_n = 0;
+    pngdec->fps_d = 1;
+  }
+
+  gst_object_unref (pngdec);
+  return TRUE;
 }
 
 static gboolean
 gst_pngdec_sink_event (GstPad * pad, GstEvent * event)
 {
   GstPngDec *pngdec;
+  gboolean res;
 
-  pngdec = GST_PNGDEC (GST_OBJECT_PARENT (pad));
+  pngdec = GST_PNGDEC (gst_pad_get_parent (pad));
 
   switch (GST_EVENT_TYPE (event)) {
-    case GST_EVENT_EOS:
+    case GST_EVENT_NEWSEGMENT:{
+      gdouble rate, arate;
+      gboolean update;
+      gint64 start, stop, position;
+      GstFormat fmt;
+
+      gst_event_parse_new_segment_full (event, &update, &rate, &arate, &fmt,
+          &start, &stop, &position);
+
+      gst_segment_set_newsegment_full (&pngdec->segment, update, rate, arate,
+          fmt, start, stop, position);
+
+      GST_LOG_OBJECT (pngdec, "NEWSEGMENT (%s)", gst_format_get_name (fmt));
+
+      if (fmt == GST_FORMAT_TIME) {
+        pngdec->need_newsegment = FALSE;
+        res = gst_pad_push_event (pngdec->srcpad, event);
+      } else {
+        gst_event_unref (event);
+        res = TRUE;
+      }
+      break;
+    }
+    case GST_EVENT_FLUSH_STOP:
+    {
       gst_pngdec_libpng_clear (pngdec);
+      gst_pngdec_libpng_init (pngdec);
+      png_set_progressive_read_fn (pngdec->png, pngdec,
+          user_info_callback, user_endrow_callback, user_end_callback);
+      pngdec->ret = GST_FLOW_OK;
+      gst_segment_init (&pngdec->segment, GST_FORMAT_UNDEFINED);
+      res = gst_pad_push_event (pngdec->srcpad, event);
+      break;
+    }
+    case GST_EVENT_EOS:
+    {
+      GST_LOG_OBJECT (pngdec, "EOS");
+      gst_pngdec_libpng_clear (pngdec);
+      pngdec->ret = GST_FLOW_UNEXPECTED;
+      res = gst_pad_push_event (pngdec->srcpad, event);
+      break;
+    }
     default:
-      return gst_pad_event_default (pad, event);
+      res = gst_pad_push_event (pngdec->srcpad, event);
+      break;
   }
+
+  gst_object_unref (pngdec);
+  return res;
 }
 
 
@@ -525,6 +688,9 @@ gst_pngdec_libpng_clear (GstPngDec * pngdec)
 
   pngdec->setup = FALSE;
 
+  pngdec->in_timestamp = GST_CLOCK_TIME_NONE;
+  pngdec->in_duration = GST_CLOCK_TIME_NONE;
+
   return TRUE;
 }
 
@@ -533,62 +699,84 @@ gst_pngdec_libpng_init (GstPngDec * pngdec)
 {
   g_return_val_if_fail (GST_IS_PNGDEC (pngdec), FALSE);
 
-  if (pngdec->setup) {
-    goto beach;
-  }
+  if (pngdec->setup)
+    return TRUE;
+
+  GST_LOG ("init libpng structures");
 
   /* initialize png struct stuff */
   pngdec->png = png_create_read_struct (PNG_LIBPNG_VER_STRING,
       (png_voidp) NULL, user_error_fn, user_warning_fn);
 
-  if (pngdec->png == NULL) {
-    GST_ELEMENT_ERROR (pngdec, LIBRARY, INIT, (NULL),
-        ("Failed to initialize png structure"));
-    goto beach;
-  }
+  if (pngdec->png == NULL)
+    goto init_failed;
 
   pngdec->info = png_create_info_struct (pngdec->png);
-  if (pngdec->info == NULL) {
-    gst_pngdec_libpng_clear (pngdec);
-    GST_ELEMENT_ERROR (pngdec, LIBRARY, INIT, (NULL),
-        ("Failed to initialize info structure"));
-    goto beach;
-  }
+  if (pngdec->info == NULL)
+    goto info_failed;
 
   pngdec->endinfo = png_create_info_struct (pngdec->png);
-  if (pngdec->endinfo == NULL) {
-    gst_pngdec_libpng_clear (pngdec);
-    GST_ELEMENT_ERROR (pngdec, LIBRARY, INIT, (NULL),
-        ("Failed to initialize endinfo structure"));
-    goto beach;
-  }
+  if (pngdec->endinfo == NULL)
+    goto endinfo_failed;
 
   pngdec->setup = TRUE;
 
-beach:
-  return pngdec->setup;
+  return TRUE;
+
+  /* ERRORS */
+init_failed:
+  {
+    GST_ELEMENT_ERROR (pngdec, LIBRARY, INIT, (NULL),
+        ("Failed to initialize png structure"));
+    return FALSE;
+  }
+info_failed:
+  {
+    gst_pngdec_libpng_clear (pngdec);
+    GST_ELEMENT_ERROR (pngdec, LIBRARY, INIT, (NULL),
+        ("Failed to initialize info structure"));
+    return FALSE;
+  }
+endinfo_failed:
+  {
+    gst_pngdec_libpng_clear (pngdec);
+    GST_ELEMENT_ERROR (pngdec, LIBRARY, INIT, (NULL),
+        ("Failed to initialize endinfo structure"));
+    return FALSE;
+  }
 }
 
 static GstStateChangeReturn
 gst_pngdec_change_state (GstElement * element, GstStateChange transition)
 {
-  GstStateChangeReturn ret = GST_STATE_CHANGE_FAILURE;
-  GstPngDec *pngdec = NULL;
+  GstStateChangeReturn ret;
+  GstPngDec *pngdec;
 
   pngdec = GST_PNGDEC (element);
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_PAUSED:
       gst_pngdec_libpng_init (pngdec);
-      break;
-    case GST_STATE_CHANGE_PAUSED_TO_READY:
-      gst_pngdec_libpng_clear (pngdec);
+      pngdec->need_newsegment = TRUE;
+      pngdec->framed = FALSE;
+      pngdec->ret = GST_FLOW_OK;
+      gst_segment_init (&pngdec->segment, GST_FORMAT_UNDEFINED);
       break;
     default:
       break;
   }
 
   ret = parent_class->change_state (element, transition);
+  if (ret != GST_STATE_CHANGE_SUCCESS)
+    return ret;
+
+  switch (transition) {
+    case GST_STATE_CHANGE_PAUSED_TO_READY:
+      gst_pngdec_libpng_clear (pngdec);
+      break;
+    default:
+      break;
+  }
 
   return ret;
 }
@@ -605,18 +793,21 @@ gst_pngdec_sink_activate_push (GstPad * sinkpad, gboolean active)
 
   if (active) {
     /* Let libpng come back here on error */
-    if (setjmp (png_jmpbuf (pngdec->png))) {
-      GST_LOG ("failed setting up libpng jumb");
-      gst_pngdec_libpng_clear (pngdec);
-      return FALSE;
-    }
+    if (setjmp (png_jmpbuf (pngdec->png)))
+      goto setup_failed;
 
     GST_LOG ("setting up progressive loading callbacks");
     png_set_progressive_read_fn (pngdec->png, pngdec,
         user_info_callback, user_endrow_callback, user_end_callback);
   }
-
   return TRUE;
+
+setup_failed:
+  {
+    GST_LOG ("failed setting up libpng jmpbuf");
+    gst_pngdec_libpng_clear (pngdec);
+    return FALSE;
+  }
 }
 
 /* this function gets called when we activate ourselves in pull mode.
