@@ -1,8 +1,8 @@
-/* -*- Mode: C; tab-width: 2; indent-tabs-mode: t; c-basic-offset: 2 -*- */
 /* GStreamer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
  *               <2000> Daniel Fischer <dan@f3c.com>
  *               <2004> Wim Taymans <wim@fluendo.com>
+ *               <2006> Zaheer Abbas Merali <zaheerabbas at merali dot org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -34,6 +34,9 @@
 #include <libavc1394/avc1394_vcr.h>
 #include <libavc1394/rom1394.h>
 #include <libraw1394/raw1394.h>
+#ifdef HAVE_LIBIEC61883
+#include <libiec61883/iec61883.h>
+#endif
 
 #include <gst/gst.h>
 
@@ -98,7 +101,8 @@ GST_ELEMENT_DETAILS ("Firewire (1394) DV video source",
     "Source/Video",
     "Source for DV video data from firewire port",
     "Erik Walthinsen <omega@temple-baptist.com>\n"
-    "Daniel Fischer <dan@f3c.com>\n" "Wim Taymans <wim@fluendo.com>");
+    "Daniel Fischer <dan@f3c.com>\n" "Wim Taymans <wim@fluendo.com>\n"
+    "Zaheer Abbas Merali <zaheerabbas at merali dot org>");
 
 static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
@@ -327,11 +331,91 @@ gst_dv1394src_get_property (GObject * object, guint prop_id, GValue * value,
   }
 }
 
+#ifdef HAVE_LIBIEC61883
+static GstDV1394Src *
+gst_dv1394src_from_raw1394handle (raw1394handle_t handle)
+{
+  iec61883_dv_t dv = (iec61883_dv_t) raw1394_get_userdata (handle);
+  iec61883_dv_fb_t dv_fb =
+      (iec61883_dv_fb_t) iec61883_dv_get_callback_data (dv);
+  return GST_DV1394SRC (iec61883_dv_fb_get_callback_data (dv_fb));
+}
+#else /* HAVE_LIBIEC61883 */
+static GstDV1394Src *
+gst_dv1394src_from_raw1394handle (raw1394handle_t handle)
+{
+  return GST_DV1394SRC (raw1394_get_userdata (handle));
+}
+#endif /* HAVE_LIBIEC61883 */
+
+#ifdef HAVE_LIBIEC61883
+static int
+gst_dv1394src_iec61883_receive (unsigned char *data, int len,
+    int complete, void *cbdata)
+{
+  GstDV1394Src *dv1394src = GST_DV1394SRC (cbdata);
+
+  if (!GST_PAD_CAPS (GST_BASE_SRC_PAD (dv1394src))) {
+    GstCaps *caps;
+    unsigned char *p = data;
+
+    // figure format (NTSC/PAL)
+    if (p[3] & 0x80) {
+      // PAL
+      dv1394src->frame_size = PAL_FRAMESIZE;
+      dv1394src->frame_rate = PAL_FRAMERATE;
+      GST_DEBUG ("PAL data");
+      caps = gst_caps_new_simple ("video/x-dv",
+          "format", G_TYPE_STRING, "PAL",
+          "systemstream", G_TYPE_BOOLEAN, TRUE, NULL);
+    } else {
+      // NTSC (untested)
+      dv1394src->frame_size = NTSC_FRAMESIZE;
+      dv1394src->frame_rate = NTSC_FRAMERATE;
+      GST_DEBUG
+          ("NTSC data [untested] - please report success/failure to <dan@f3c.com>");
+      caps = gst_caps_new_simple ("video/x-dv",
+          "format", G_TYPE_STRING, "NTSC",
+          "systemstream", G_TYPE_BOOLEAN, TRUE, NULL);
+    }
+    gst_pad_set_caps (GST_BASE_SRC_PAD (dv1394src), caps);
+    gst_caps_unref (caps);
+  }
+  dv1394src->frame = NULL;
+  if ((dv1394src->frame_sequence + 1) % (dv1394src->skip +
+          dv1394src->consecutive) < dv1394src->consecutive) {
+    if (complete && len == dv1394src->frame_size) {
+      gint64 i64;
+      guint8 *bufdata;
+      GstBuffer *buf;
+      GstFormat format;
+
+      buf = gst_buffer_new_and_alloc (dv1394src->frame_size);
+
+      /* fill in offset, duration, timestamp */
+      GST_BUFFER_OFFSET (buf) = dv1394src->frame_sequence;
+      format = GST_FORMAT_TIME;
+      gst_dv1394src_convert (GST_BASE_SRC_PAD (dv1394src), GST_FORMAT_DEFAULT,
+          GST_BUFFER_OFFSET (buf), &format, &i64);
+      GST_BUFFER_TIMESTAMP (buf) = i64;
+      gst_dv1394src_convert (GST_BASE_SRC_PAD (dv1394src), GST_FORMAT_DEFAULT,
+          1, &format, &i64);
+      GST_BUFFER_DURATION (buf) = i64;
+      bufdata = GST_BUFFER_DATA (buf);
+      memcpy (bufdata, data, len);
+      dv1394src->buf = buf;
+    }
+  }
+  dv1394src->frame_sequence++;
+  return 0;
+}
+
+#else
 static int
 gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
     quadlet_t * data)
 {
-  GstDV1394Src *dv1394src = GST_DV1394SRC (raw1394_get_userdata (handle));
+  GstDV1394Src *dv1394src = gst_dv1394src_from_raw1394handle (handle);
 
   if (len > 16) {
     /*
@@ -340,6 +424,7 @@ gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
        this file in CVS.
      */
     unsigned char *p = (unsigned char *) &data[3];
+
     int section_type = p[0] >> 5;       /* section type is in bits 5 - 7 */
     int dif_sequence = p[1] >> 4;       /* dif sequence number is in bits 4 - 7 */
     int dif_block = p[2];
@@ -347,7 +432,6 @@ gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
     /* if we are at the beginning of a frame, 
        we set buf=frame, and alloc a new buffer for frame
      */
-
     if (section_type == 0 && dif_sequence == 0) {       // dif header
       if (!GST_PAD_CAPS (GST_BASE_SRC_PAD (dv1394src))) {
         GstCaps *caps;
@@ -386,7 +470,6 @@ gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
           gst_buffer_unref (dv1394src->frame);
         }
       }
-      dv1394src->frame = NULL;
       if ((dv1394src->frame_sequence + 1) % (dv1394src->skip +
               dv1394src->consecutive) < dv1394src->consecutive) {
         GstFormat format;
@@ -449,7 +532,7 @@ gst_dv1394src_iso_receive (raw1394handle_t handle, int channel, size_t len,
 
   return 0;
 }
-
+#endif
 /*
  * When an ieee1394 bus reset happens, usually a device has been removed
  * or added.  We send a message on the message bus with the node count 
@@ -470,7 +553,7 @@ gst_dv1394src_bus_reset (raw1394handle_t handle, unsigned int generation)
   gint current_device_change;
   gint i;
 
-  src = GST_DV1394SRC (raw1394_get_userdata (handle));
+  src = gst_dv1394src_from_raw1394handle (handle);
 
   GST_INFO_OBJECT (src, "have bus reset");
 
@@ -537,17 +620,17 @@ gst_dv1394src_create (GstPushSrc * psrc, GstBuffer ** buf)
       else
         goto error_while_polling;
     }
-
-    if (pollfds[1].revents) {
+    if (G_UNLIKELY (pollfds[1].revents)) {
       char command;
 
       if (pollfds[1].revents & POLLIN)
         READ_COMMAND (dv1394src, command, res);
 
       goto told_to_stop;
-    } else if (pollfds[0].revents & POLLIN) {
+    } else if (G_LIKELY (pollfds[0].revents & POLLIN)) {
       /* shouldn't block in theory */
       raw1394_loop_iterate (dv1394src->handle);
+
       if (dv1394src->buf)
         break;
     }
@@ -561,7 +644,6 @@ gst_dv1394src_create (GstPushSrc * psrc, GstBuffer ** buf)
 
   *buf = dv1394src->buf;
   dv1394src->buf = NULL;
-
   return GST_FLOW_OK;
 
 error_while_polling:
@@ -598,23 +680,24 @@ gst_dv1394src_discover_avc_node (GstDV1394Src * src)
     /* open the port */
     handle = raw1394_new_handle ();
     if (!handle) {
-      g_warning ("raw1394 - failed to get handle: %s.\n", strerror (errno));
+      GST_WARNING ("raw1394 - failed to get handle: %s.\n", strerror (errno));
       continue;
     }
     if ((n_ports = raw1394_get_port_info (handle, pinf, 16)) < 0) {
-      g_warning ("raw1394 - failed to get port info: %s.\n", strerror (errno));
+      GST_WARNING ("raw1394 - failed to get port info: %s.\n",
+          strerror (errno));
       goto next;
     }
 
     /* tell raw1394 which host adapter to use */
     if (raw1394_set_port (handle, j) < 0) {
-      g_warning ("raw1394 - failed to set set port: %s.\n", strerror (errno));
+      GST_WARNING ("raw1394 - failed to set set port: %s.\n", strerror (errno));
       goto next;
     }
 
     /* now loop over all the nodes */
     for (i = 0; i < raw1394_get_nodecount (handle); i++) {
-      /* are we looking for an explicit GUID */
+      /* are we looking for an explicit GUID ? */
       if (src->guid != 0) {
         if (src->guid == rom1394_get_guid (handle, i)) {
           node = i;
@@ -626,9 +709,9 @@ gst_dv1394src_discover_avc_node (GstDV1394Src * src)
       } else {
         rom1394_directory rom_dir;
 
-        /* select first AV/C Tape Reccorder Player node */
+        /* select first AV/C Tape Recorder Player node */
         if (rom1394_get_directory (handle, i, &rom_dir) < 0) {
-          g_warning ("error reading config rom directory for node %d\n", i);
+          GST_WARNING ("error reading config rom directory for node %d\n", i);
           continue;
         }
         if ((rom1394_get_node_type (&rom_dir) == ROM1394_NODE_TYPE_AVC) &&
@@ -676,8 +759,6 @@ gst_dv1394src_start (GstBaseSrc * bsrc)
       goto no_handle;
   }
 
-  raw1394_set_userdata (src->handle, src);
-
   src->num_ports = raw1394_get_port_info (src->handle, src->pinfo, 16);
 
   if (src->num_ports == 0)
@@ -686,24 +767,53 @@ gst_dv1394src_start (GstBaseSrc * bsrc)
   if (src->use_avc || src->port == -1)
     src->avc_node = gst_dv1394src_discover_avc_node (src);
 
-  if (raw1394_set_port (src->handle, src->port) < 0)
+  /* lets destroy handle and create one on port
+     this is more reliable than setting port on
+     the existing handle */
+  raw1394_destroy_handle (src->handle);
+  src->handle = raw1394_new_handle_on_port (src->port);
+  if (!src->handle)
     goto cannot_set_port;
 
+  raw1394_set_userdata (src->handle, src);
+  raw1394_set_bus_reset_handler (src->handle, gst_dv1394src_bus_reset);
+
+#ifdef HAVE_LIBIEC61883
+  if ((src->iec61883dv =
+          iec61883_dv_fb_init (src->handle,
+              gst_dv1394src_iec61883_receive, src)) == NULL)
+    goto cannot_initialise_dv;
+
+#else
   raw1394_set_iso_handler (src->handle, src->channel,
       gst_dv1394src_iso_receive);
-  raw1394_set_bus_reset_handler (src->handle, gst_dv1394src_bus_reset);
+#endif
 
   GST_DEBUG_OBJECT (src, "successfully opened up 1394 connection");
   src->connected = TRUE;
+
+#ifdef HAVE_LIBIEC61883
+  if (iec61883_dv_fb_start (src->iec61883dv, src->channel) != 0)
+    goto cannot_start;
+#else
   if (raw1394_start_iso_rcv (src->handle, src->channel) < 0)
     goto cannot_start;
+#endif
 
   if (src->use_avc) {
+    raw1394handle_t avc_handle = raw1394_new_handle_on_port (src->port);
+
     /* start the VCR */
-    if (!avc1394_vcr_is_recording (src->handle, src->avc_node)
-        && avc1394_vcr_is_playing (src->handle, src->avc_node)
-        != AVC1394_VCR_OPERAND_PLAY_FORWARD)
-      avc1394_vcr_play (src->handle, src->avc_node);
+    if (avc_handle) {
+      if (!avc1394_vcr_is_recording (avc_handle, src->avc_node)
+          && avc1394_vcr_is_playing (avc_handle, src->avc_node)
+          != AVC1394_VCR_OPERAND_PLAY_FORWARD)
+        avc1394_vcr_play (avc_handle, src->avc_node);
+      raw1394_destroy_handle (avc_handle);
+    } else {
+      GST_WARNING_OBJECT (src, "Starting VCR via avc1394 failed: %s",
+          g_strerror (errno));
+    }
   }
 
   return TRUE;
@@ -748,6 +858,14 @@ cannot_start:
         ("can't start 1394 iso receive"));
     return FALSE;
   }
+#ifdef HAVE_LIBIEC61883
+cannot_initialise_dv:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
+        ("can't initialise iec61883 dv"));
+    return FALSE;
+  }
+#endif
 }
 
 static gboolean
@@ -759,19 +877,28 @@ gst_dv1394src_stop (GstBaseSrc * bsrc)
   close (WRITE_SOCKET (src));
   READ_SOCKET (src) = -1;
   WRITE_SOCKET (src) = -1;
-
+#ifdef HAVE_LIBIEC61883
+  iec61883_dv_fb_close (src->iec61883dv);
+#else
   raw1394_stop_iso_rcv (src->handle, src->channel);
-  if (src->use_avc) {
-    /* pause the VCR */
-    if (!avc1394_vcr_is_recording (src->handle, src->avc_node)
-        && (avc1394_vcr_is_playing (src->handle, src->avc_node)
-            != AVC1394_VCR_OPERAND_PLAY_FORWARD_PAUSE))
-      avc1394_vcr_pause (src->handle, src->avc_node);
-  }
+#endif
 
-  if (src->use_avc)
-    /* stop the VCR */
-    avc1394_vcr_stop (src->handle, src->avc_node);
+  if (src->use_avc) {
+    raw1394handle_t avc_handle = raw1394_new_handle_on_port (src->port);
+
+    /* pause and stop the VCR */
+    if (avc_handle) {
+      if (!avc1394_vcr_is_recording (avc_handle, src->avc_node)
+          && (avc1394_vcr_is_playing (avc_handle, src->avc_node)
+              != AVC1394_VCR_OPERAND_PLAY_FORWARD_PAUSE))
+        avc1394_vcr_pause (avc_handle, src->avc_node);
+      avc1394_vcr_stop (avc_handle, src->avc_node);
+      raw1394_destroy_handle (avc_handle);
+    } else {
+      GST_WARNING_OBJECT (src, "Starting VCR via avc1394 failed: %s",
+          g_strerror (errno));
+    }
+  }
 
   raw1394_destroy_handle (src->handle);
 
