@@ -31,7 +31,7 @@
  * <title>Example launch line</title>
  * <para>
  * <programlisting>
- * gst-launch -v -m audiotestsrc ! autoaudiosink
+ * gst-launch -v -m audiotestsrc ! audioconvert ! audioresample ! autoaudiosink
  * </programlisting>
  * </para>
  * </refsect2>
@@ -52,29 +52,34 @@ gst_auto_audio_sink_change_state (GstElement * element,
 
 GST_BOILERPLATE (GstAutoAudioSink, gst_auto_audio_sink, GstBin, GST_TYPE_BIN);
 
+static const GstElementDetails gst_auto_audio_sink_details =
+GST_ELEMENT_DETAILS ("Auto audio sink",
+    "Sink/Audio",
+    "Wrapper audio sink for automatically detected audio sink",
+    "Ronald Bultje <rbultje@ronald.bitfreak.net>");
+
+static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS_ANY);
+
 static void
 gst_auto_audio_sink_base_init (gpointer klass)
 {
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
-  const GstElementDetails gst_auto_audio_sink_details =
-      GST_ELEMENT_DETAILS ("Auto audio sink",
-      "Sink/Audio",
-      "Wrapper audio sink for automatically detected audio sink",
-      "Ronald Bultje <rbultje@ronald.bitfreak.net>");
-  GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
-      GST_PAD_SINK,
-      GST_PAD_ALWAYS,
-      GST_STATIC_CAPS_ANY);
 
   gst_element_class_add_pad_template (eklass,
       gst_static_pad_template_get (&sink_template));
+
   gst_element_class_set_details (eklass, &gst_auto_audio_sink_details);
 }
 
 static void
 gst_auto_audio_sink_class_init (GstAutoAudioSinkClass * klass)
 {
-  GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
+  GstElementClass *eklass;
+
+  eklass = GST_ELEMENT_CLASS (klass);
 
   eklass->change_state = GST_DEBUG_FUNCPTR (gst_auto_audio_sink_change_state);
 }
@@ -150,12 +155,32 @@ gst_auto_audio_sink_compare_ranks (GstPluginFeature * f1, GstPluginFeature * f2)
       gst_plugin_feature_get_name (f1));
 }
 
+static GstElement *
+gst_auto_audio_sink_create_element_with_pretty_name (GstAutoAudioSink * sink,
+    GstElementFactory * factory)
+{
+  GstElement *element;
+  gchar *name, *marker;
+
+  marker = g_strdup (GST_PLUGIN_FEATURE (factory)->name);
+  if (g_str_has_suffix (marker, "sink"))
+    marker[strlen (marker) - 4] = '\0';
+  if (g_str_has_prefix (marker, "gst"))
+    g_memmove (marker, marker + 3, strlen (marker + 3) + 1);
+  name = g_strdup_printf ("%s-actual-sink-%s", GST_OBJECT_NAME (sink), marker);
+  g_free (marker);
+
+  element = gst_element_factory_create (factory, name);
+  g_free (name);
+
+  return element;
+}
+
 static GstElement *
 gst_auto_audio_sink_find_best (GstAutoAudioSink * sink)
 {
   GList *list, *item;
   GstElement *choice = NULL;
-  gboolean ss = TRUE;
   GstMessage *message = NULL;
   GSList *errors = NULL;
   GstBus *bus = gst_bus_new ();
@@ -164,75 +189,43 @@ gst_auto_audio_sink_find_best (GstAutoAudioSink * sink)
       (GstPluginFeatureFilter) gst_auto_audio_sink_factory_filter, FALSE, sink);
   list = g_list_sort (list, (GCompareFunc) gst_auto_audio_sink_compare_ranks);
 
-  /* FIXME:
-   * - soundservers have no priority yet.
-   * - soundserversinks should only be chosen if already running, or if
-   *    the user explicitly wants this to run... That is not easy.
-   */
+  /* We don't treat sound server sinks special. Our policy is that sound
+   * server sinks that have a rank must not auto-spawn a daemon under any
+   * circumstances, so there's nothing for us to worry about here */
+  GST_LOG_OBJECT (sink, "Trying to find usable audio devices ...");
 
-  while (1) {
-    GST_DEBUG_OBJECT (sink, "Trying to find %s",
-        ss ? "soundservers" : "audio devices");
+  for (item = list; item != NULL; item = item->next) {
+    GstElementFactory *f = GST_ELEMENT_FACTORY (item->data);
+    GstElement *el;
 
-    for (item = list; item != NULL; item = item->next) {
-      GstElementFactory *f = GST_ELEMENT_FACTORY (item->data);
-      GstElement *el;
+    if ((el = gst_auto_audio_sink_create_element_with_pretty_name (sink, f))) {
+      GstStateChangeReturn ret;
 
-      if ((el = gst_element_factory_create (f, "actual-sink"))) {
-        /* FIXME: no element actually has this property as far as I can tell.
-         * also, this is a nasty uncheckable way of supporting something that
-         * amounts to being an interface. */
-        gboolean has_p = g_object_class_find_property (G_OBJECT_GET_CLASS (el),
-            "soundserver-running") ? TRUE : FALSE;
-
-        if (ss == has_p) {
-          if (ss) {
-            gboolean r;
-
-            g_object_get (el, "soundserver-running", &r, NULL);
-            if (r) {
-              GST_DEBUG_OBJECT (sink, "%s - soundserver is running",
-                  GST_PLUGIN_FEATURE (f)->name);
-            } else {
-              GST_DEBUG_OBJECT (sink, "%s - Soundserver is not running",
-                  GST_PLUGIN_FEATURE (f)->name);
-              goto next;
-            }
-          }
-          GST_DEBUG_OBJECT (sink, "Testing %s", GST_PLUGIN_FEATURE (f)->name);
-          gst_element_set_bus (el, bus);
-          if (gst_element_set_state (el,
-                  GST_STATE_READY) == GST_STATE_CHANGE_SUCCESS) {
-            gst_element_set_state (el, GST_STATE_NULL);
-            GST_DEBUG_OBJECT (sink, "This worked!");
-            choice = el;
-            goto done;
-          } else {
-            /* collect all error messages */
-            while ((message = gst_bus_pop (bus))) {
-              if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_ERROR) {
-                GST_DEBUG_OBJECT (sink, "appending error message %"
-                    GST_PTR_FORMAT, message);
-                errors = g_slist_append (errors, message);
-              } else {
-                gst_message_unref (message);
-              }
-            }
-            gst_element_set_state (el, GST_STATE_NULL);
-          }
-        }
-
-      next:
-        gst_object_unref (el);
+      GST_DEBUG_OBJECT (sink, "Testing %s", GST_PLUGIN_FEATURE (f)->name);
+      gst_element_set_bus (el, bus);
+      ret = gst_element_set_state (el, GST_STATE_READY);
+      if (ret == GST_STATE_CHANGE_SUCCESS) {
+        gst_element_set_state (el, GST_STATE_NULL);
+        GST_DEBUG_OBJECT (sink, "This worked!");
+        choice = el;
+        break;
       }
-    }
 
-    if (!ss)
-      break;
-    ss = FALSE;
+      /* collect all error messages */
+      while ((message = gst_bus_pop (bus))) {
+        if (GST_MESSAGE_TYPE (message) == GST_MESSAGE_ERROR) {
+          GST_DEBUG_OBJECT (sink, "error message %" GST_PTR_FORMAT, message);
+          errors = g_slist_append (errors, message);
+        } else {
+          gst_message_unref (message);
+        }
+      }
+
+      gst_element_set_state (el, GST_STATE_NULL);
+      gst_object_unref (el);
+    }
   }
 
-done:
   GST_DEBUG_OBJECT (sink, "done trying");
   if (!choice) {
     if (errors) {
@@ -242,9 +235,13 @@ done:
       GST_DEBUG_OBJECT (sink, "reposting message %p", errors->data);
       gst_element_post_message (GST_ELEMENT (sink), GST_MESSAGE (errors->data));
     } else {
-      /* general fallback */
-      GST_ELEMENT_ERROR (sink, LIBRARY, INIT, (NULL),
-          ("Failed to find a supported audio sink"));
+      /* send warning message to application and use a fakesink */
+      GST_ELEMENT_WARNING (sink, RESOURCE, NOT_FOUND, (NULL),
+          ("Failed to find a usable audio sink"));
+      choice = gst_element_factory_make ("fakesink", "fake-audio-sink");
+      if (g_object_class_find_property (G_OBJECT_GET_CLASS (choice), "sync"))
+        g_object_set (choice, "sync", TRUE, NULL);
+      gst_element_set_state (choice, GST_STATE_READY);
     }
   }
   gst_object_unref (bus);
@@ -304,6 +301,8 @@ gst_auto_audio_sink_change_state (GstElement * element,
   }
 
   ret = GST_ELEMENT_CLASS (parent_class)->change_state (element, transition);
+  if (ret == GST_STATE_CHANGE_FAILURE)
+    return ret;
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_NULL:
