@@ -46,6 +46,7 @@
 #include <gst/base/gsttypefindhelper.h>
 #include <gst/gst-i18n-plugin.h>
 #include <gst/tag/tag.h>
+#include <string.h>
 
 #include "gstid3demux.h"
 #include "id3tags.h"
@@ -281,6 +282,7 @@ gst_id3demux_add_srcpad (GstID3Demux * id3demux, GstCaps * new_caps)
         id3demux->src_caps);
 
     gst_object_ref (id3demux->srcpad);
+    gst_pad_set_active (id3demux->srcpad, TRUE);
     if (!(gst_element_add_pad (GST_ELEMENT (id3demux), id3demux->srcpad)))
       return FALSE;
     gst_element_no_more_pads (GST_ELEMENT (id3demux));
@@ -305,6 +307,10 @@ gst_id3demux_remove_srcpad (GstID3Demux * id3demux)
   return res;
 };
 
+/* will return FALSE if buffer is beyond end of data; will return TRUE
+ * if buffer was trimmed successfully or didn't need trimming, but may
+ * also return TRUE and set *buf_ref to NULL if the buffer was before
+ * the start of the data */
 static gboolean
 gst_id3demux_trim_buffer (GstID3Demux * id3demux, GstBuffer ** buf_ref)
 {
@@ -328,7 +334,7 @@ gst_id3demux_trim_buffer (GstID3Demux * id3demux, GstBuffer ** buf_ref)
 
       if (out_offset >= v1tag_offset) {
         GST_DEBUG_OBJECT (id3demux, "Buffer is past the end of the data");
-        goto no_out_buffer;
+        goto no_out_buffer_end;
       }
 
       if (out_offset + out_size > v1tag_offset) {
@@ -343,7 +349,7 @@ gst_id3demux_trim_buffer (GstID3Demux * id3demux, GstBuffer ** buf_ref)
     if (out_offset <= id3demux->strip_start) {
       if (out_offset + out_size <= id3demux->strip_start) {
         GST_DEBUG_OBJECT (id3demux, "Buffer is before the start of the data");
-        goto no_out_buffer;
+        goto no_out_buffer_start;
       }
 
       trim_start = id3demux->strip_start - out_offset;
@@ -381,10 +387,19 @@ gst_id3demux_trim_buffer (GstID3Demux * id3demux, GstBuffer ** buf_ref)
   }
 
   return TRUE;
-no_out_buffer:
-  gst_buffer_unref (buf);
-  *buf_ref = NULL;
-  return TRUE;
+
+no_out_buffer_end:
+  {
+    gst_buffer_unref (buf);
+    *buf_ref = NULL;
+    return FALSE;
+  }
+no_out_buffer_start:
+  {
+    gst_buffer_unref (buf);
+    *buf_ref = NULL;
+    return TRUE;
+  }
 }
 
 static GstFlowReturn
@@ -404,11 +419,25 @@ gst_id3demux_chain (GstPad * pad, GstBuffer * buf)
 
   switch (id3demux->state) {
     case GST_ID3DEMUX_READID3V2:
+      if (GST_BUFFER_SIZE (id3demux->collect) < 3)
+        break;                  /* Go get more data first */
+
+      /* need to set offset of first buffer to 0 or trimming won't work */
+      if (!GST_BUFFER_OFFSET_IS_VALID (id3demux->collect) &&
+          memcmp (GST_BUFFER_DATA (id3demux->collect), "ID3", 3) == 0) {
+        GST_WARNING_OBJECT (id3demux, "Fixing up first buffer without offset");
+        id3demux->collect =
+            gst_buffer_make_metadata_writable (id3demux->collect);
+        GST_BUFFER_OFFSET (id3demux->collect) = 0;
+      }
+
       /* If we receive a buffer that's from the middle of the file, 
        * we can't read tags so move to typefinding */
-      if (GST_BUFFER_OFFSET (id3demux->collect) != 0) {
+      if (!GST_BUFFER_OFFSET_IS_VALID (id3demux->collect) ||
+          GST_BUFFER_OFFSET (id3demux->collect) != 0) {
         GST_DEBUG_OBJECT (id3demux,
-            "Received buffer from non-zero offset. Can't read tags");
+            "Received buffer with non-zero offset %" G_GINT64_FORMAT
+            ". Can't read tags", GST_BUFFER_OFFSET (id3demux->collect));
       } else {
         ID3TagsResult tag_result;
 
@@ -444,7 +473,7 @@ gst_id3demux_chain (GstPad * pad, GstBuffer * buf)
       typefind_buf = id3demux->collect;
       gst_buffer_ref (typefind_buf);
       if (!gst_id3demux_trim_buffer (id3demux, &typefind_buf))
-        return GST_FLOW_ERROR;
+        return GST_FLOW_UNEXPECTED;
 
       if (typefind_buf == NULL)
         break;                  /* Still need more data */
@@ -507,7 +536,7 @@ gst_id3demux_chain (GstPad * pad, GstBuffer * buf)
         outbuf = id3demux->collect;
         id3demux->collect = NULL;
         if (!gst_id3demux_trim_buffer (id3demux, &outbuf))
-          return GST_FLOW_ERROR;
+          return GST_FLOW_UNEXPECTED;
       }
       if (outbuf) {
         if (G_UNLIKELY (id3demux->srcpad == NULL)) {
@@ -600,37 +629,21 @@ gst_id3demux_get_property (GObject * object, guint prop_id,
 static gboolean
 id3demux_get_upstream_size (GstID3Demux * id3demux)
 {
-  GstQuery *query;
-  GstPad *peer = NULL;
   GstFormat format;
   gint64 result;
-  gboolean res = FALSE;
 
   /* Short-cut if we already queried upstream */
   if (id3demux->upstream_size > 0)
     return TRUE;
 
-  if ((peer = gst_pad_get_peer (id3demux->sinkpad)) == NULL)
+  format = GST_FORMAT_BYTES;
+  if (!gst_pad_query_peer_duration (id3demux->sinkpad, &format, &result) ||
+      result < 0) {
     return FALSE;
-
-  query = gst_query_new_duration (GST_FORMAT_BYTES);
-  gst_query_set_duration (query, GST_FORMAT_BYTES, -1);
-
-  if (!gst_pad_query (peer, query))
-    goto out;
-
-  gst_query_parse_duration (query, &format, &result);
-  gst_query_unref (query);
-
-  if (format != GST_FORMAT_BYTES || result == -1)
-    goto out;
+  }
 
   id3demux->upstream_size = result;
-  res = TRUE;
-
-out:
-  gst_object_unref (peer);
-  return res;
+  return TRUE;
 }
 
 static gboolean
@@ -949,25 +962,34 @@ gst_id3demux_read_range (GstID3Demux * id3demux,
   if (!id3demux_get_upstream_size (id3demux))
     return GST_FLOW_ERROR;
 
-  if (in_offset + length >= id3demux->upstream_size - id3demux->strip_end)
+  if (in_offset + length >= id3demux->upstream_size - id3demux->strip_end) {
+    if (in_offset + id3demux->strip_end >= id3demux->upstream_size)
+      return GST_FLOW_UNEXPECTED;
     in_length = id3demux->upstream_size - id3demux->strip_end - in_offset;
-  else
+  } else {
     in_length = length;
+  }
 
   ret = gst_pad_pull_range (id3demux->sinkpad, in_offset, in_length, buffer);
-
   if (ret == GST_FLOW_OK && *buffer) {
     if (!gst_id3demux_trim_buffer (id3demux, buffer))
-      goto error;
+      goto read_beyond_end;
+
+    /* this should only happen in streaming mode */
+    g_assert (*buffer != NULL);
   }
 
   return ret;
-error:
-  if (*buffer != NULL) {
-    gst_buffer_unref (buffer);
-    *buffer = NULL;
+
+read_beyond_end:
+  {
+    GST_DEBUG_OBJECT (id3demux, "attempted read beyond end of file");
+    if (*buffer != NULL) {
+      gst_buffer_unref (buffer);
+      *buffer = NULL;
+    }
+    return GST_FLOW_UNEXPECTED;
   }
-  return GST_FLOW_ERROR;
 }
 
 static GstFlowReturn
@@ -1084,6 +1106,11 @@ plugin_init (GstPlugin * plugin)
 
   gst_tag_register_musicbrainz_tags ();
 
+  /* ensure private tag is registered */
+  gst_tag_register (GST_ID3_DEMUX_TAG_ID3V2_FRAME, GST_TAG_FLAG_META,
+      GST_TYPE_BUFFER, "ID3v2 frame", "unparsed id3v2 tag frame",
+      gst_tag_merge_use_first);
+
   return gst_element_register (plugin, "id3demux",
       GST_RANK_PRIMARY, GST_TYPE_ID3DEMUX);
 }
