@@ -9,7 +9,12 @@
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more 
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #ifdef HAVE_CONFIG_H
@@ -23,8 +28,8 @@
 
 /* elementfactory information */
 static const GstElementDetails gst_rtp_mp4vdepay_details =
-GST_ELEMENT_DETAILS ("RTP packet parser",
-    "Codec/Depayr/Network",
+GST_ELEMENT_DETAILS ("RTP packet depayloader",
+    "Codec/Depayloader/Network",
     "Extracts MPEG4 video from RTP packets (RFC 3016)",
     "Wim Taymans <wim@fluendo.com>");
 
@@ -106,8 +111,6 @@ gst_rtp_mp4v_depay_class_init (GstRtpMP4VDepayClass * klass)
 
   gstbasertpdepayload_class = (GstBaseRTPDepayloadClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gstbasertpdepayload_class->process = gst_rtp_mp4v_depay_process;
   gstbasertpdepayload_class->set_caps = gst_rtp_mp4v_depay_setcaps;
 
@@ -299,5 +302,5 @@ gboolean
 gst_rtp_mp4v_depay_plugin_init (GstPlugin * plugin)
 {
   return gst_element_register (plugin, "rtpmp4vdepay",
-      GST_RANK_NONE, GST_TYPE_RTP_MP4V_DEPAY);
+      GST_RANK_MARGINAL, GST_TYPE_RTP_MP4V_DEPAY);
 }
