@@ -46,6 +46,8 @@
  * <programlisting>
  * gst-launch -v udpsrc port=0 ! fakesink
  * </programlisting>
+ * After setting the udpsrc to PAUSED, the allocated port can be obtained by
+ * reading the port property.
  * </para>
  * <para>
  * udpsrc can read from multicast groups by setting the multicast_group property
@@ -62,6 +64,13 @@
  * using SDP or other means. 
  * </para>
  * <para>
+ * The "buffer" property is used to change the default kernel buffer sizes used for
+ * receiving packets. The buffer size may be increased for high-volume connections,
+ * or may be decreased to limit the possible backlog of incoming data.
+ * The system places an absolute limit on these values, on Linux, for example, the
+ * default buffer size is typically 50K and can be increased to maximally 100K.
+ * </para>
+ * <para>
  * The udpsrc is always a live source. It does however not provide a GstClock, this
  * is left for upstream elements such as an RTP session manager or demuxer (such
  * as an MPEG demuxer).
@@ -70,23 +79,44 @@
  * udpsrc implements a GstURIHandler interface that handles udp://host:port type
  * URIs.
  * </para>
+ * <para>
+ * If the <link linkend="GstUDPSrc--timeout">timeout property</link> is set to a
+ * value bigger than 0, udpsrc will generate an element message named
+ * <classname>&quot;GstUDPSrcTimeout&quot;</classname>:
+ * if no data was recieved in the given timeout.
+ * The message's structure contains one field:
+ * <itemizedlist>
+ * <listitem>
+ *   <para>
+ *   #guint64
+ *   <classname>&quot;timeout&quot;</classname>: the timeout in microseconds that
+ *   expired when waiting for data.
+ *   </para>
+ * </listitem>
+ * </itemizedlist>
+ * The message is typically used to detect that no UDP arrives in the receiver
+ * because it is blocked by a firewall.
+ * </para>
+ * <para>
+ * Last reviewed on 2006-09-29 (0.10.5)
+ * </para>
  * </refsect2>
  */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
 #include "gstudpsrc.h"
 #include <unistd.h>
-#include <sys/ioctl.h>
+#include <stdlib.h>
+
 #include <gst/netbuffer/gstnetbuffer.h>
 
 #ifdef HAVE_FIONREAD_IN_SYS_FILIO
 #include <sys/filio.h>
 #endif
 
-GST_DEBUG_CATEGORY (udpsrc_debug);
+GST_DEBUG_CATEGORY_STATIC (udpsrc_debug);
 #define GST_CAT_DEFAULT (udpsrc_debug)
 
 /* the select call is also performed on the control sockets, that way
@@ -97,10 +127,10 @@ GST_DEBUG_CATEGORY (udpsrc_debug);
 #define WRITE_SOCKET(src)      src->control_sock[1]
 #define READ_SOCKET(src)       src->control_sock[0]
 
-#define SEND_COMMAND(src, command)              \
-G_STMT_START {                                  \
-  unsigned char c; c = command;                 \
-  write (WRITE_SOCKET(src), &c, 1);             \
+#define SEND_COMMAND(src, command, res)          \
+G_STMT_START {                                   \
+  unsigned char c; c = command;                  \
+  res = write (WRITE_SOCKET(src), &c, 1);        \
 } G_STMT_END
 
 #define READ_COMMAND(src, command, res)         \
@@ -117,13 +147,16 @@ static const GstElementDetails gst_udpsrc_details =
 GST_ELEMENT_DETAILS ("UDP packet receiver",
     "Source/Network",
     "Receive data over the network via UDP",
-    "Wim Taymans <wim@fluendo.com>");
+    "Wim Taymans <wim@fluendo.com>\n"
+    "Thijs Vermeir <thijs.vermeir@barco.com>");
 
 #define UDP_DEFAULT_PORT                4951
 #define UDP_DEFAULT_MULTICAST_GROUP     "0.0.0.0"
 #define UDP_DEFAULT_URI                 "udp://"UDP_DEFAULT_MULTICAST_GROUP":"G_STRINGIFY(UDP_DEFAULT_PORT)
 #define UDP_DEFAULT_CAPS                NULL
 #define UDP_DEFAULT_SOCKFD              -1
+#define UDP_DEFAULT_BUFFER_SIZE		0
+#define UDP_DEFAULT_TIMEOUT             0
 
 enum
 {
@@ -132,8 +165,9 @@ enum
   PROP_MULTICAST_GROUP,
   PROP_URI,
   PROP_CAPS,
-  PROP_SOCKFD
-      /* FILL ME */
+  PROP_SOCKFD,
+  PROP_BUFFER_SIZE,
+  PROP_TIMEOUT
 };
 
 static void gst_udpsrc_uri_handler_init (gpointer g_iface, gpointer iface_data);
@@ -143,6 +177,7 @@ static GstFlowReturn gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf);
 static gboolean gst_udpsrc_start (GstBaseSrc * bsrc);
 static gboolean gst_udpsrc_stop (GstBaseSrc * bsrc);
 static gboolean gst_udpsrc_unlock (GstBaseSrc * bsrc);
+static void gst_udpsrc_finalize (GObject * object);
 
 static void gst_udpsrc_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -190,13 +225,14 @@ gst_udpsrc_class_init (GstUDPSrcClass * klass)
 
   gobject_class->set_property = gst_udpsrc_set_property;
   gobject_class->get_property = gst_udpsrc_get_property;
+  gobject_class->finalize = gst_udpsrc_finalize;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_PORT,
-      g_param_spec_int ("port", "port",
+      g_param_spec_int ("port", "Port",
           "The port to receive the packets from, 0=allocate", 0, G_MAXUINT16,
           UDP_DEFAULT_PORT, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_MULTICAST_GROUP,
-      g_param_spec_string ("multicast_group", "multicast_group",
+      g_param_spec_string ("multicast_group", "Multicast Group",
           "The Address of multicast group to join", UDP_DEFAULT_MULTICAST_GROUP,
           G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_URI,
@@ -207,9 +243,17 @@ gst_udpsrc_class_init (GstUDPSrcClass * klass)
       g_param_spec_boxed ("caps", "Caps",
           "The caps of the source pad", GST_TYPE_CAPS, G_PARAM_READWRITE));
   g_object_class_install_property (gobject_class, PROP_SOCKFD,
-      g_param_spec_int ("sockfd", "socket handle",
+      g_param_spec_int ("sockfd", "Socket Handle",
           "Socket to use for UDP reception. (-1 == allocate)",
           -1, G_MAXINT, UDP_DEFAULT_SOCKFD, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_BUFFER_SIZE,
+      g_param_spec_int ("buffer-size", "Buffer Size",
+          "Size of the kernel receive buffer in bytes, 0=default", 0, G_MAXINT,
+          UDP_DEFAULT_BUFFER_SIZE, G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), PROP_TIMEOUT,
+      g_param_spec_uint64 ("timeout", "Timeout",
+          "Post a message after timeout microseconds (0 = disabled)", 0,
+          G_MAXUINT64, UDP_DEFAULT_TIMEOUT, G_PARAM_READWRITE));
 
   gstbasesrc_class->start = gst_udpsrc_start;
   gstbasesrc_class->stop = gst_udpsrc_stop;
@@ -222,11 +266,33 @@ gst_udpsrc_class_init (GstUDPSrcClass * klass)
 static void
 gst_udpsrc_init (GstUDPSrc * udpsrc, GstUDPSrcClass * g_class)
 {
+  WSA_STARTUP (udpsrc);
+
   gst_base_src_set_live (GST_BASE_SRC (udpsrc), TRUE);
   udpsrc->port = UDP_DEFAULT_PORT;
   udpsrc->sock = UDP_DEFAULT_SOCKFD;
   udpsrc->multi_group = g_strdup (UDP_DEFAULT_MULTICAST_GROUP);
   udpsrc->uri = g_strdup (UDP_DEFAULT_URI);
+  udpsrc->buffer_size = UDP_DEFAULT_BUFFER_SIZE;
+  udpsrc->timeout = UDP_DEFAULT_TIMEOUT;
+
+  udpsrc->control_sock[0] = -1;
+  udpsrc->control_sock[1] = -1;
+}
+
+static void
+gst_udpsrc_finalize (GObject * object)
+{
+  GstUDPSrc *udpsrc;
+
+  udpsrc = GST_UDPSRC (object);
+
+  if (udpsrc->caps)
+    gst_caps_unref (udpsrc->caps);
+  g_free (udpsrc->multi_group);
+  g_free (udpsrc->uri);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 static GstCaps *
@@ -253,31 +319,69 @@ gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
   guint max_sock;
   gchar *pktdata;
   gint pktsize;
+
+#ifdef G_OS_UNIX
   gint readsize;
+#elif defined G_OS_WIN32
+  gulong readsize;
+#endif
   gint ret;
   gboolean try_again;
 
   udpsrc = GST_UDPSRC (psrc);
 
-  FD_ZERO (&read_fds);
-  FD_SET (udpsrc->sock, &read_fds);
-  FD_SET (READ_SOCKET (udpsrc), &read_fds);
-  max_sock = MAX (udpsrc->sock, READ_SOCKET (udpsrc));
-
   do {
     gboolean stop;
+    struct timeval timeval, *timeout;
+
+    FD_ZERO (&read_fds);
+    FD_SET (udpsrc->sock, &read_fds);
+#ifndef G_OS_WIN32
+    FD_SET (READ_SOCKET (udpsrc), &read_fds);
+#endif
+    max_sock = MAX (udpsrc->sock, READ_SOCKET (udpsrc));
 
     try_again = FALSE;
     stop = FALSE;
 
-    GST_LOG_OBJECT (udpsrc, "doing select");
-    ret = select (max_sock + 1, &read_fds, NULL, NULL, NULL);
+    GST_LOG_OBJECT (udpsrc, "doing select, timeout %" G_GUINT64_FORMAT,
+        udpsrc->timeout);
+
+    if (udpsrc->timeout > 0) {
+      timeval.tv_sec = udpsrc->timeout / 1000000;
+      timeval.tv_usec = udpsrc->timeout % 1000000;
+      timeout = &timeval;
+    } else {
+      timeout = NULL;
+    }
+
+#ifdef G_OS_WIN32
+    if (((max_sock + 1) != READ_SOCKET (udpsrc)) ||
+        ((max_sock + 1) != WRITE_SOCKET (udpsrc))) {
+      ret = select (max_sock + 1, &read_fds, NULL, NULL, timeout);
+    } else {
+      ret = 1;
+    }
+#else
+    ret = select (max_sock + 1, &read_fds, NULL, NULL, timeout);
+#endif
     GST_LOG_OBJECT (udpsrc, "select returned %d", ret);
-    if (ret <= 0) {
+    if (ret < 0) {
+#ifdef G_OS_WIN32
+      if (WSAGetLastError () != WSAEINTR)
+        goto select_error;
+#else
       if (errno != EAGAIN && errno != EINTR)
         goto select_error;
-      else
-        try_again = TRUE;
+#endif
+      try_again = TRUE;
+    } else if (ret == 0) {
+      /* timeout, post element message */
+      gst_element_post_message (GST_ELEMENT_CAST (udpsrc),
+          gst_message_new_element (GST_OBJECT_CAST (udpsrc),
+              gst_structure_new ("GstUDPSrcTimeout",
+                  "timeout", G_TYPE_UINT64, udpsrc->timeout, NULL)));
+      try_again = TRUE;
     } else {
       if (FD_ISSET (READ_SOCKET (udpsrc), &read_fds)) {
         /* got control message */
@@ -286,7 +390,7 @@ gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
           int res;
 
           READ_COMMAND (udpsrc, command, res);
-          if (res < 0) {
+          if (res <= 0) {
             GST_LOG_OBJECT (udpsrc, "no more commands");
             /* no more commands */
             break;
@@ -311,8 +415,9 @@ gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
       goto stopped;
   } while (try_again);
 
-  /* ask how much is available for reading on the socket */
-  if ((ret = ioctl (udpsrc->sock, FIONREAD, &readsize)) < 0)
+  /* ask how much is available for reading on the socket, this is exactly one
+   * UDP packet. */
+  if ((ret = IOCTL_SOCKET (udpsrc->sock, FIONREAD, &readsize)) < 0)
     goto ioctl_failed;
 
   GST_LOG_OBJECT (udpsrc, "ioctl says %d bytes available", readsize);
@@ -320,8 +425,8 @@ gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
   pktdata = g_malloc (readsize);
   pktsize = readsize;
 
-  len = sizeof (struct sockaddr);
   while (TRUE) {
+    len = sizeof (struct sockaddr);
     ret = recvfrom (udpsrc->sock, pktdata, pktsize,
         0, (struct sockaddr *) &tmpaddr, &len);
     if (ret < 0) {
@@ -331,6 +436,7 @@ gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
       break;
   }
 
+  /* special case buffer so receivers can also track the address */
   outbuf = gst_netbuffer_new ();
   GST_BUFFER_DATA (outbuf) = (guint8 *) pktdata;
   GST_BUFFER_MALLOCDATA (outbuf) = (guint8 *) pktdata;
@@ -339,14 +445,15 @@ gst_udpsrc_create (GstPushSrc * psrc, GstBuffer ** buf)
   gst_netaddress_set_ip4_address (&outbuf->from, tmpaddr.sin_addr.s_addr,
       tmpaddr.sin_port);
 
-  gst_buffer_set_caps (GST_BUFFER (outbuf), udpsrc->caps);
+  gst_buffer_set_caps (GST_BUFFER_CAST (outbuf), udpsrc->caps);
 
   GST_LOG_OBJECT (udpsrc, "read %d bytes", readsize);
 
-  *buf = GST_BUFFER (outbuf);
+  *buf = GST_BUFFER_CAST (outbuf);
 
   return GST_FLOW_OK;
 
+  /* ERRORS */
 select_error:
   {
     GST_ELEMENT_ERROR (udpsrc, RESOURCE, READ, (NULL),
@@ -413,11 +520,12 @@ gst_udpsrc_set_uri (GstUDPSrc * src, const gchar * uri)
 
   return TRUE;
 
+  /* ERRORS */
 wrong_protocol:
   {
     g_free (protocol);
     GST_ELEMENT_ERROR (src, RESOURCE, READ, (NULL),
-        ("error parsing uri %s: wrong protocol", uri));
+        ("error parsing uri %s: wrong protocol (%s != udp)", uri, protocol));
     return FALSE;
   }
 }
@@ -429,6 +537,9 @@ gst_udpsrc_set_property (GObject * object, guint prop_id, const GValue * value,
   GstUDPSrc *udpsrc = GST_UDPSRC (object);
 
   switch (prop_id) {
+    case PROP_BUFFER_SIZE:
+      udpsrc->buffer_size = g_value_get_int (value);
+      break;
     case PROP_PORT:
       udpsrc->port = g_value_get_int (value);
       gst_udpsrc_update_uri (udpsrc);
@@ -439,7 +550,7 @@ gst_udpsrc_set_property (GObject * object, guint prop_id, const GValue * value,
       if (g_value_get_string (value) == NULL)
         udpsrc->multi_group = g_strdup (UDP_DEFAULT_MULTICAST_GROUP);
       else
-        udpsrc->multi_group = g_strdup (g_value_get_string (value));
+        udpsrc->multi_group = g_value_dup_string (value);
       gst_udpsrc_update_uri (udpsrc);
       break;
     case PROP_URI:
@@ -468,6 +579,9 @@ gst_udpsrc_set_property (GObject * object, guint prop_id, const GValue * value,
       udpsrc->sock = g_value_get_int (value);
       GST_DEBUG ("setting SOCKFD to %d", udpsrc->sock);
       break;
+    case PROP_TIMEOUT:
+      udpsrc->timeout = g_value_get_uint64 (value);
+      break;
     default:
       break;
   }
@@ -480,6 +594,9 @@ gst_udpsrc_get_property (GObject * object, guint prop_id, GValue * value,
   GstUDPSrc *udpsrc = GST_UDPSRC (object);
 
   switch (prop_id) {
+    case PROP_BUFFER_SIZE:
+      g_value_set_int (value, udpsrc->buffer_size);
+      break;
     case PROP_PORT:
       g_value_set_int (value, udpsrc->port);
       break;
@@ -495,6 +612,9 @@ gst_udpsrc_get_property (GObject * object, guint prop_id, GValue * value,
     case PROP_SOCKFD:
       g_value_set_int (value, udpsrc->sock);
       break;
+    case PROP_TIMEOUT:
+      g_value_set_uint64 (value, udpsrc->timeout);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -512,18 +632,31 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
   int port;
   GstUDPSrc *src;
   gint ret;
+  int rcvsize;
 
   src = GST_UDPSRC (bsrc);
 
+#ifdef G_OS_WIN32
+  GST_DEBUG_OBJECT (src, "creating pipe");
+
+  /* This should work on UNIX too. PF_UNIX sockets replaced with pipe */
+  /* pipe( CONTROL_SOCKETS(src) ) */
+  if ((ret = pipe (CONTROL_SOCKETS (src))) < 0)
+    goto no_socket_pair;
+#else
   GST_DEBUG_OBJECT (src, "creating socket pair");
   if ((ret = socketpair (PF_UNIX, SOCK_STREAM, 0, CONTROL_SOCKETS (src))) < 0)
     goto no_socket_pair;
 
   fcntl (READ_SOCKET (src), F_SETFL, O_NONBLOCK);
   fcntl (WRITE_SOCKET (src), F_SETFL, O_NONBLOCK);
+#endif
+
+  if (!inet_aton (src->multi_group, &(src->multi_addr.imr_multiaddr)))
+    src->multi_addr.imr_multiaddr.s_addr = 0;
 
   if (src->sock == -1) {
-    if ((ret = socket (AF_INET, SOCK_DGRAM, 0)) < 0)
+    if ((ret = socket (PF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0)
       goto no_socket;
 
     src->sock = ret;
@@ -537,29 +670,57 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
     memset (&src->myaddr, 0, sizeof (src->myaddr));
     src->myaddr.sin_family = AF_INET;   /* host byte order */
     src->myaddr.sin_port = htons (src->port);   /* short, network byte order */
-    src->myaddr.sin_addr.s_addr = INADDR_ANY;
+
+    if (src->multi_addr.imr_multiaddr.s_addr)
+      src->myaddr.sin_addr.s_addr = src->multi_addr.imr_multiaddr.s_addr;
+    else
+      src->myaddr.sin_addr.s_addr = INADDR_ANY;
 
     GST_DEBUG_OBJECT (src, "binding on port %d", src->port);
-    if ((ret =
-            bind (src->sock, (struct sockaddr *) &src->myaddr,
+    if ((ret = bind (src->sock, (struct sockaddr *) &src->myaddr,
                 sizeof (src->myaddr))) < 0)
       goto bind_error;
   }
 
-  if (inet_aton (src->multi_group, &(src->multi_addr.imr_multiaddr))) {
-    if (src->multi_addr.imr_multiaddr.s_addr) {
-      src->multi_addr.imr_interface.s_addr = INADDR_ANY;
-      if ((ret =
-              setsockopt (src->sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
-                  &src->multi_addr, sizeof (src->multi_addr))) < 0)
-        goto membership;
-    }
+  if (src->multi_addr.imr_multiaddr.s_addr) {
+    src->multi_addr.imr_interface.s_addr = INADDR_ANY;
+    if ((ret =
+            setsockopt (src->sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+                &src->multi_addr, sizeof (src->multi_addr))) < 0)
+      goto membership;
   }
 
   len = sizeof (my_addr);
   if ((ret = getsockname (src->sock, (struct sockaddr *) &my_addr, &len)) < 0)
     goto getsockname_error;
 
+  len = sizeof (rcvsize);
+  if (src->buffer_size != 0) {
+    rcvsize = src->buffer_size;
+
+    GST_DEBUG_OBJECT (src, "setting udp buffer of %d bytes", rcvsize);
+    /* set buffer size, Note that on Linux this is typically limited to a
+     * maximum of around 100K. Also a minimum of 128 bytes is required on
+     * Linux. */
+    ret = setsockopt (src->sock, SOL_SOCKET, SO_RCVBUF, (void *) &rcvsize, len);
+    if (ret != 0)
+      goto udpbuffer_error;
+  }
+
+  /* read the value of the receive buffer. Note that on linux this returns 2x the
+   * value we set because the kernel allocates extra memory for metadata.
+   * The default on Linux is about 100K (which is about 50K without metadata) */
+  ret = getsockopt (src->sock, SOL_SOCKET, SO_RCVBUF, (void *) &rcvsize, &len);
+  if (ret == 0)
+    GST_DEBUG_OBJECT (src, "have udp buffer of %d bytes", rcvsize);
+  else
+    GST_DEBUG_OBJECT (src, "could not get udp buffer size");
+
+  bc_val = 1;
+  if ((ret = setsockopt (src->sock, SOL_SOCKET, SO_BROADCAST, &bc_val,
+              sizeof (bc_val))) < 0)
+    goto no_broadcast;
+
   port = ntohs (my_addr.sin_port);
   GST_DEBUG_OBJECT (src, "bound, on port %d", port);
   if (port != src->port) {
@@ -568,12 +729,6 @@ gst_udpsrc_start (GstBaseSrc * bsrc)
     g_object_notify (G_OBJECT (src), "port");
   }
 
-  bc_val = 1;
-  if ((ret =
-          setsockopt (src->sock, SOL_SOCKET, SO_BROADCAST, &bc_val,
-              sizeof (bc_val))) < 0)
-    goto no_broadcast;
-
   src->myaddr.sin_port = htons (src->port + 1);
 
   return TRUE;
@@ -593,7 +748,7 @@ no_socket:
   }
 setsockopt_error:
   {
-    close (src->sock);
+    CLOSE_SOCKET (src->sock);
     src->sock = -1;
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("setsockopt failed %d: %s (%d)", ret, g_strerror (errno), errno));
@@ -601,7 +756,7 @@ setsockopt_error:
   }
 bind_error:
   {
-    close (src->sock);
+    CLOSE_SOCKET (src->sock);
     src->sock = -1;
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("bind failed %d: %s (%d)", ret, g_strerror (errno), errno));
@@ -609,7 +764,7 @@ bind_error:
   }
 membership:
   {
-    close (src->sock);
+    CLOSE_SOCKET (src->sock);
     src->sock = -1;
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("could add membership %d: %s (%d)", ret, g_strerror (errno), errno));
@@ -617,15 +772,24 @@ membership:
   }
 getsockname_error:
   {
-    close (src->sock);
+    CLOSE_SOCKET (src->sock);
     src->sock = -1;
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("getsockname failed %d: %s (%d)", ret, g_strerror (errno), errno));
     return FALSE;
   }
+udpbuffer_error:
+  {
+    CLOSE_SOCKET (src->sock);
+    src->sock = -1;
+    GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
+        ("Could not create a buffer of the size requested, %d: %s (%d)", ret,
+            g_strerror (errno), errno));
+    return FALSE;
+  }
 no_broadcast:
   {
-    close (src->sock);
+    CLOSE_SOCKET (src->sock);
     src->sock = -1;
     GST_ELEMENT_ERROR (src, RESOURCE, SETTINGS, (NULL),
         ("could not configure socket for broadcast %d: %s (%d)", ret,
@@ -638,11 +802,13 @@ static gboolean
 gst_udpsrc_unlock (GstBaseSrc * bsrc)
 {
   GstUDPSrc *src;
+  gint res;
 
   src = GST_UDPSRC (bsrc);
 
   GST_DEBUG ("sending stop command");
-  SEND_COMMAND (src, CONTROL_STOP);
+  SEND_COMMAND (src, CONTROL_STOP, res);
+  GST_DEBUG ("sent stop command %d", res);
 
   return TRUE;
 }
@@ -654,11 +820,14 @@ gst_udpsrc_stop (GstBaseSrc * bsrc)
 
   src = GST_UDPSRC (bsrc);
 
+  GST_DEBUG ("stopping, closing sockets");
+
   if (src->sock != -1) {
-    close (src->sock);
+    CLOSE_SOCKET (src->sock);
     src->sock = -1;
   }
 
+  /* pipes on WIN32 else sockets */
   if (src->control_sock[0] != -1) {
     close (src->control_sock[0]);
     src->control_sock[0] = -1;
@@ -667,12 +836,15 @@ gst_udpsrc_stop (GstBaseSrc * bsrc)
     close (src->control_sock[1]);
     src->control_sock[1] = -1;
   }
+
+  WSA_CLEANUP (src);
+
   return TRUE;
 }
 
 /*** GSTURIHANDLER INTERFACE *************************************************/
 
-static guint
+static GstURIType
 gst_udpsrc_uri_get_type (void)
 {
   return GST_URI_SRC;
