@@ -27,7 +27,7 @@
 #include <liboil/liboil.h>
 #include <string.h>
 
-GST_DEBUG_CATEGORY (videobox_debug);
+GST_DEBUG_CATEGORY_STATIC (videobox_debug);
 #define GST_CAT_DEFAULT videobox_debug
 
 #define GST_TYPE_VIDEO_BOX \
@@ -38,7 +38,7 @@ GST_DEBUG_CATEGORY (videobox_debug);
   (G_TYPE_CHECK_CLASS_CAST((klass),GST_TYPE_VIDEO_BOX,GstVideoBoxClass))
 #define GST_IS_VIDEO_BOX(obj) \
   (G_TYPE_CHECK_INSTANCE_TYPE((obj),GST_TYPE_VIDEO_BOX))
-#define GST_IS_VIDEO_BOX_CLASS(obj) \
+#define GST_IS_VIDEO_BOX_CLASS(klass) \
   (G_TYPE_CHECK_CLASS_TYPE((klass),GST_TYPE_VIDEO_BOX))
 
 typedef struct _GstVideoBox GstVideoBox;
@@ -49,6 +49,7 @@ typedef enum
   VIDEO_BOX_FILL_BLACK,
   VIDEO_BOX_FILL_GREEN,
   VIDEO_BOX_FILL_BLUE,
+  VIDEO_BOX_FILL_LAST
 }
 GstVideoBoxFill;
 
@@ -102,8 +103,8 @@ enum
   PROP_BOTTOM,
   PROP_FILL_TYPE,
   PROP_ALPHA,
-  PROP_BORDER_ALPHA,
-  /* FILL ME */
+  PROP_BORDER_ALPHA
+      /* FILL ME */
 };
 
 static GstStaticPadTemplate gst_video_box_src_template =
@@ -144,7 +145,7 @@ static GType
 gst_video_box_fill_get_type (void)
 {
   static GType video_box_fill_type = 0;
-  static GEnumValue video_box_fill[] = {
+  static const GEnumValue video_box_fill[] = {
     {VIDEO_BOX_FILL_BLACK, "Black", "black"},
     {VIDEO_BOX_FILL_GREEN, "Colorkey green", "green"},
     {VIDEO_BOX_FILL_BLUE, "Colorkey blue", "blue"},
@@ -212,10 +213,11 @@ gst_video_box_class_init (GstVideoBoxClass * klass)
           "Alpha value of the border", 0.0, 1.0, DEFAULT_BORDER_ALPHA,
           G_PARAM_READWRITE));
 
-  trans_class->transform_caps = gst_video_box_transform_caps;
-  trans_class->set_caps = gst_video_box_set_caps;
-  trans_class->get_unit_size = gst_video_box_get_unit_size;
-  trans_class->transform = gst_video_box_transform;
+  trans_class->transform_caps =
+      GST_DEBUG_FUNCPTR (gst_video_box_transform_caps);
+  trans_class->set_caps = GST_DEBUG_FUNCPTR (gst_video_box_set_caps);
+  trans_class->get_unit_size = GST_DEBUG_FUNCPTR (gst_video_box_get_unit_size);
+  trans_class->transform = GST_DEBUG_FUNCPTR (gst_video_box_transform);
 
   GST_DEBUG_CATEGORY_INIT (videobox_debug, "videobox", 0,
       "Resizes a video by adding borders or cropping");
@@ -440,7 +442,8 @@ gst_video_box_get_unit_size (GstBaseTransform * trans, GstCaps * caps,
   guint32 fourcc;
   gint width, height;
 
-  g_return_val_if_fail (size, FALSE);
+  g_assert (size);
+
   video_box = GST_VIDEO_BOX (trans);
 
   structure = gst_caps_get_structure (caps, 0);
@@ -463,9 +466,9 @@ gst_video_box_get_unit_size (GstBaseTransform * trans, GstCaps * caps,
   return TRUE;
 }
 
-static int yuv_colors_Y[] = { 16, 150, 29 };
-static int yuv_colors_U[] = { 128, 46, 255 };
-static int yuv_colors_V[] = { 128, 21, 107 };
+static const guint8 yuv_colors_Y[VIDEO_BOX_FILL_LAST] = { 16, 150, 29 };
+static const guint8 yuv_colors_U[VIDEO_BOX_FILL_LAST] = { 128, 46, 255 };
+static const guint8 yuv_colors_V[VIDEO_BOX_FILL_LAST] = { 128, 21, 107 };
 
 static void
 gst_video_box_copy_plane_i420 (GstVideoBox * video_box, guint8 * src,
