@@ -140,7 +140,7 @@ ximageutil_xcontext_get (GstElement * parent, const gchar * display_name)
   xcontext = g_new0 (GstXContext, 1);
 
   xcontext->disp = XOpenDisplay (display_name);
-  GST_DEBUG_OBJECT (parent, "opened display 0x%x", xcontext->disp);
+  GST_DEBUG_OBJECT (parent, "opened display %p", xcontext->disp);
   if (!xcontext->disp) {
     g_free (xcontext);
     return NULL;
@@ -210,6 +210,10 @@ ximageutil_xcontext_get (GstElement * parent, const gchar * display_name)
       xcontext->g_mask_output >>= 8;
       xcontext->b_mask_output >>= 8;
     }
+  } else {
+    xcontext->r_mask_output = xcontext->visual->red_mask;
+    xcontext->g_mask_output = xcontext->visual->green_mask;
+    xcontext->b_mask_output = xcontext->visual->blue_mask;
   }
 
   return xcontext;
